@@ -5,16 +5,22 @@
 ##' Add a primary or foreign key to a table column.
 ##'
 ##' @param conn A connection object.
-##' @param name A character string specifying a PostgreSQL table name.
+##' @param name A character string, or a character vector, specifying
+##'     a PostgreSQL table name.
 ##' @param colname A character string specifying the name of the
-##'     column to which the key will be assign.
+##'     column to which the key will be assign; alternatively, a
+##'     character vector specifying the name of the columns for keys
+##'     spanning more than one column.
 ##' @param type The type of the key, either \code{"primary"} or
 ##'     \code{"foreign"}
 ##' @param reference A character string specifying a foreign table
-##'     name to which the foreign key will be associated.
+##'     name to which the foreign key will be associated (ignored if
+##'     \code{type == "primary"}).
 ##' @param colref A character string specifying the name of the
 ##'     primary key in the foreign table to which the foreign key will
-##'     be associated.
+##'     be associated; alternatively, a character vector specifying
+##'     the name of the columns for keys spanning more than one column
+##'     (ignored if \code{type == "primary"}).
 ##' @param display Logical. Whether to display the query (defaults to
 ##'     \code{TRUE}).
 ##' @param exec Logical. Whether to execute the query (defaults to
@@ -25,35 +31,44 @@
 ##' @author Mathieu Basille \email{basille@@ufl.edu}
 ##' @export
 ##' @examples
-##' ## examples use a dummy connection from DBI package
-##' conn<-DBI::ANSI()
-##' dbAddKey(conn, name = c("schema", "table"), colname = "id", type = "foreign",
-##'     reference = c("flu", "bla"), colref = "id", exec = FALSE)
-
+##' ## Examples use a dummy connection from DBI package
+##' conn <- DBI::ANSI()
+##'
+##' ## Primary key
+##' dbAddKey(conn, name = c("sch1", "tbl1"), colname = "id1", exec = FALSE)
+##'
+##' ## Primary key using multiple columns
+##' dbAddKey(conn, name = c("sch1", "tbl1"), colname = c("id1", "id2",
+##'     "id3"), exec = FALSE)
+##'
+##' ## Foreign key
+##' dbAddKey(conn, name = c("sch1", "tbl1"), colname = "id", type = "foreign",
+##'     reference = c("sch2", "tbl2"), colref = "id", exec = FALSE)
+##'
+##' ## Foreign key using multiple columns
+##' dbAddKey(conn, name = c("sch1", "tbl1"), colname = c("id1", "id2"),
+##'     type = "foreign", reference = c("sch2", "tbl2"), colref = c("id3",
+##'         "id4"), exec = FALSE)
 dbAddKey <- function(conn, name, colname, type = c("primary",
     "foreign"), reference, colref, display = TRUE, exec = TRUE) {
-    ## Check and prepare the schema.name
-    name <- dbTableNameFix(conn,name)
+    ## Check and prepare the schema.name and column name
+    name <- dbTableNameFix(conn, name)
     nameque <- paste(name, collapse = ".")
-    colname<-DBI::dbQuoteIdentifier(conn,colname)
-    if (missing(colref)) {
-      # If no reference, empty string
-      colref <- "" 
-      } else { 
-      colref<-DBI::dbQuoteIdentifier(conn,colref)
-      }
-    ## 'type' in upper case
+    colname <- paste(DBI::dbQuoteIdentifier(conn, colname), collapse = ", ")
+    ## Check 'type' and set it to upper case
     type <- toupper(match.arg(type))
-    ## If no reference, empty string
-    if (missing(reference)) {
+    ## If primary key, both 'reference' and 'colref' are ignored
+    ## (empty strings)
+    if (type == "PRIMARY") {
+        colref <- ""
         references <- ""
-    } else {
-        ## Else, check and prepare the schema.name of the reference
-        ## table
-        reference <- dbTableNameFix(conn,reference)
-        reftable <- paste(reference, collapse = ".")
-        references <- paste0(" REFERENCES ", reftable, " (",
-            colref, ")")
+    ## If foreign key, check identifiers for 'reference' and 'colref'
+    } else if (type == "FOREIGN") {
+        colref <- paste(DBI::dbQuoteIdentifier(conn, colref),
+            collapse = ", ")
+        reference <- dbTableNameFix(conn, reference)
+        references <- paste0(" REFERENCES ", paste(reference,
+            collapse = "."), " (", colref, ")")
     }
     ## Build the query
     tmp.query <- paste0("ALTER TABLE ", nameque, " ADD ", type,
@@ -62,17 +77,16 @@ dbAddKey <- function(conn, name, colname, type = c("primary",
     if (display) {
         message(paste0("Query ", ifelse(exec, "", "not "), "executed:"))
         message(tmp.query)
-        #message("--")
     }
-    ## Execute the query
+    ## Execute the query and return TRUE
     if (exec) {
         dbConnCheck(conn)
         dbSendQuery(conn, tmp.query)
+        return(TRUE)
     }
-    ## Return TRUE
-    if(exec) return(TRUE)
 }
 
+
 ## dbAsDate
 
 ##' Converts to timestamp.
@@ -326,6 +340,7 @@ dbDrop <- function(conn, name, type = c("table", "schema", "view", "materialized
     if(exec) return(TRUE)
 }
 
+
 ## dbIndex
 
 ##' Create an index.
@@ -334,11 +349,14 @@ dbDrop <- function(conn, name, type = c("table", "schema", "view", "materialized
 ##'
 ##' @param conn A connection object.
 ##' @param name A character string specifying a PostgreSQL table name.
-##' @param colname A character string specifying the name of the
-##'     column to which the key will be associated.
+##' @param colname A character string, or a character vector
+##'     specifying the name of the column to which the key will be
+##'     associated; alternatively, a character vector specifying the
+##'     name of the columns to build the index.
 ##' @param idxname A character string specifying the name of the index
-##'     to be created. By default, this is the name of the table
-##'     (without the schema) suffixed by \code{_idx}.
+##'     to be created. By default, this uses the name of the table
+##'     (without the schema) and the name of the columns as follows:
+##'     \code{<table_name>_<column_names>_idx}.
 ##' @param unique Logical. Causes the system to check for duplicate
 ##'     values in the table when the index is created (if data already
 ##'     exist) and each time data is added. Attempts to insert or
@@ -348,7 +366,7 @@ dbDrop <- function(conn, name, type = c("table", "schema", "view", "materialized
 ##'     index. Choices are \code{"btree"}, \code{"hash"},
 ##'     \code{"rtree"}, and \code{"gist"}. The default method is
 ##'     \code{"btree"}, although \code{"gist"} should be the index of
-##'     choice for Post GIS spatial types (geometry, geography,
+##'     choice for PostGIS spatial types (geometry, geography,
 ##'     raster).
 ##' @param display Logical. Whether to display the query (defaults to
 ##'     \code{TRUE}).
@@ -362,26 +380,30 @@ dbDrop <- function(conn, name, type = c("table", "schema", "view", "materialized
 ##' @author Mathieu Basille \email{basille@@ufl.edu}
 ##' @export
 ##' @examples
-##' ## examples use a dummy connection from DBI package
-##' conn<-DBI::ANSI()
-##' dbIndex(conn,name = c("schema", "table"), colname = "geom", method = "gist",
+##' ## Examples use a dummy connection from DBI package
+##' conn <- DBI::ANSI()
+##'
+##' ## GIST index
+##' dbIndex(conn, name = c("sch", "tbl"), colname = "geom", method = "gist",
 ##'     exec = FALSE)
-
+##'
+##' ## Regular BTREE index on multiple columns
+##' dbIndex(conn, name = c("sch", "tbl"), colname = c("col1", "col2",
+##'     "col3"), exec = FALSE)
 dbIndex <- function(conn, name, colname, idxname, unique = FALSE,
     method = c("btree", "hash", "rtree", "gist"), display = TRUE,
     exec = TRUE) {
-    ## Check and prepare the schema.name and column name
-    name <- dbTableNameFix(conn,name)
-    nameque <- paste(name, collapse = ".")
-    colname<-DBI::dbQuoteIdentifier(conn,colname)
     ## Check and prepare the name of the index
     if (missing(idxname)) {
-        idxname <- DBI::dbQuoteIdentifier(conn,
-            paste(gsub('"','',name[length(name)]), gsub('"','',colname), "idx",
-            sep = "_"))
-        } else {
-        idxname<-DBI::dbQuoteIdentifier(conn,idxname)      
-            }
+        idxname <- DBI::dbQuoteIdentifier(conn, paste(name[length(name)],
+            paste(colname, collapse = "_"), "idx", sep = "_"))
+    } else {
+        idxname <- DBI::dbQuoteIdentifier(conn, idxname)
+    }
+    ## Check and prepare the schema.name and column name
+    name <- dbTableNameFix(conn, name)
+    nameque <- paste(name, collapse = ".")
+    colname <- paste(DBI::dbQuoteIdentifier(conn, colname), collapse = ", ")
     ## Argument UNIQUE
     unique <- ifelse(unique, "UNIQUE ", "")
     ## Check and prepare the method for the index
@@ -395,17 +417,17 @@ dbIndex <- function(conn, name, colname, idxname, unique = FALSE,
     if (display) {
         message(paste0("Query ", ifelse(exec, "", "not "), "executed:"))
         message(tmp.query)
-        #message("--")
+        message("--")
     }
-    ## Execute the query
+    ## Execute the query and return TRUE
     if (exec) {
         dbConnCheck(conn)
         dbSendQuery(conn, tmp.query)
+        return(TRUE)
     }
-    ## Return true
-    if(exec) return(TRUE)
 }
 
+
 ## dbSchema
 
 ##' Check and create schema.
@@ -558,4 +580,4 @@ dbVacuum <- function(conn, name, full = FALSE, verbose = FALSE,
     }
     ## Return true
     if(exec) return(TRUE)
-}
\ No newline at end of file
+}
